import { Controller, UsePipes } from '@nestjs/common';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { RpcValidationPipe } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { MatchPaginatedResponseDto } from '../dtos/match.paginated.response.dto';
import { QueryBus } from '@nestjs/cqrs';
import { MatchRequestDto } from './dtos/match.request.dto';
import { MatchQuery } from '@modules/matcher/core/application/queries/match/match.query';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class MatchGrpcController {
  constructor(private readonly queryBus: QueryBus) {}

  @GrpcMethod('MatcherService', 'Match')
  async match(data: MatchRequestDto): Promise<MatchPaginatedResponseDto> {
    try {
      const matches = await this.queryBus.execute(new MatchQuery(data));
      return {
        data: matches,
        page: 1,
        perPage: 5,
        total: 1,
      };
    } catch (e) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
